package com.qmth.distributed.print.business.templete.execute;

import cn.hutool.core.date.DateUtil;
import com.alibaba.fastjson.JSON;
import com.qmth.distributed.print.business.bean.dto.SyncExamTaskDto;
import com.qmth.distributed.print.business.bean.marking.GroupInfo;
import com.qmth.distributed.print.business.bean.params.SyncDataParam;
import com.qmth.distributed.print.business.entity.ExamPaperStructure;
import com.qmth.distributed.print.business.entity.TBSyncTask;
import com.qmth.distributed.print.business.service.DataSyncService;
import com.qmth.distributed.print.business.service.TBSyncTaskService;
import com.qmth.distributed.print.business.service.TSyncStmmsExamService;
import com.qmth.teachcloud.common.entity.SysUser;
import com.qmth.teachcloud.common.enums.ExceptionResultEnum;
import com.qmth.teachcloud.common.enums.PushTypeEnum;
import com.qmth.teachcloud.common.service.SysOrgService;
import com.qmth.teachcloud.common.sync.CloudMarkingTaskUtils;
import com.qmth.teachcloud.common.util.ServletUtil;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.context.annotation.Lazy;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.*;
import java.util.stream.Collectors;

/**
 * 异步同步云阅卷
 */
@Service
public class AsyncCloudMarkingTaskService {

    private final static Logger log = LoggerFactory.getLogger(AsyncCloudMarkingTaskService.class);

    @Resource
    private SysOrgService sysOrgService;

    @Resource
    private TSyncStmmsExamService tSyncStmmsExamService;

    @Lazy
    @Resource
    private DataSyncService dataSyncService;

    @Resource
    private TBSyncTaskService tbSyncTaskService;

    @Resource
    private CloudMarkingTaskUtils cloudMarkingTaskUtils;

    /**
     * 同步考试、考生、科目、题卡
     *
     * @param schoolId 学校ID
     */
    public void syncExamAndStudentAndCard(Long schoolId, SyncDataParam syncDataParam) {
        SysUser sysUser = (SysUser) ServletUtil.getRequestUser();
        String thirdRelateName = syncDataParam.getThirdRelateName();
        String examTime = Objects.isNull(syncDataParam.getExamTime()) ? null : DateUtil.format(new Date(syncDataParam.getExamTime()), CloudMarkingTaskUtils.DATE_FORMAT);

        // 分学院
        if (cloudMarkingTaskUtils.isCollegeMode(schoolId)) {
            // 按学院code进行分组，并按学院进行循环同步
            Map<String, List<SyncExamTaskDto>> byCollegeCodeMap = syncDataParam.getList().stream().collect(Collectors.groupingBy(SyncExamTaskDto::getCollegeCode));
            for (Map.Entry<String, List<SyncExamTaskDto>> entry : byCollegeCodeMap.entrySet()) {
                // 本行为，若为按学校同步，则后续方法都传null,方便校验
                String orgCode = entry.getKey();
                List<SyncExamTaskDto> syncExamTaskDtos = entry.getValue();
                List<TBSyncTask> tbSyncTaskList = new ArrayList<>();
                for (SyncExamTaskDto syncExamTaskDto : syncExamTaskDtos) {
                    TBSyncTask tbSyncTask = tbSyncTaskService.saveTask(schoolId, Long.valueOf(syncExamTaskDto.getExamTaskId()), orgCode, thirdRelateName, examTime, PushTypeEnum.EXAM_PUSH, JSON.toJSONString(syncExamTaskDto));
                    tbSyncTaskList.add(tbSyncTask);
                }
                dataSyncService.syncExamAndStudentAndCard(orgCode, thirdRelateName, examTime, tbSyncTaskList, sysUser);
            }
        } else {
            List<TBSyncTask> tbSyncTaskList = new ArrayList<>();
            for (SyncExamTaskDto syncExamTaskDto : syncDataParam.getList()) {
                TBSyncTask tbSyncTask = tbSyncTaskService.saveTask(schoolId, Long.valueOf(syncExamTaskDto.getExamTaskId()), null, thirdRelateName, examTime, PushTypeEnum.EXAM_PUSH, JSON.toJSONString(syncExamTaskDto));
                tbSyncTaskList.add(tbSyncTask);
            }
            dataSyncService.syncExamAndStudentAndCard(null, thirdRelateName, examTime, tbSyncTaskList, sysUser);
        }
    }

    /**
     * 同步试卷结构、分组信息、评卷员绑定关系
     *
     * @param examPaperStructure 试卷结构对象
     */
    public void syncPaperStructureAndGroup(ExamPaperStructure examPaperStructure) {
        if (examPaperStructure == null) {
            throw ExceptionResultEnum.ERROR.exception("未找到同步数据");
        }
        TBSyncTask tbSyncTask = tbSyncTaskService.saveTask(examPaperStructure.getSchoolId(), examPaperStructure.getId(), PushTypeEnum.STRUCTURE_GROUP_PUSH, null);
        dataSyncService.syncPaperStructureAndGroup(examPaperStructure, tbSyncTask);
    }

    /**
     * 同步试卷文件、标签文件
     *
     * @param examPaperStructure 试卷结构对象
     */
    public void syncPaperAndAnswer(ExamPaperStructure examPaperStructure) {
        if (examPaperStructure == null) {
            throw ExceptionResultEnum.ERROR.exception("请先设置评卷参数");
        }
        TBSyncTask tbSyncTask = tbSyncTaskService.saveTask(examPaperStructure.getSchoolId(), examPaperStructure.getId(), PushTypeEnum.PAPER_ANSWER_FILE_PUSH, null);
        dataSyncService.syncPaperAndAnswer(examPaperStructure, tbSyncTask);
    }

    /**
     * 同步客观题答案
     *
     * @param examPaperStructure 试卷结构对象
     */
    public void syncObjectiveStructure(ExamPaperStructure examPaperStructure) {
        if (examPaperStructure == null) {
            throw ExceptionResultEnum.ERROR.exception("请先设置评卷参数");
        }
        TBSyncTask tbSyncTask = tbSyncTaskService.saveTask(examPaperStructure.getSchoolId(), examPaperStructure.getId(), PushTypeEnum.OBJECTIVE_ANSWER_PUSH, null);
        dataSyncService.syncObjectiveStructure(examPaperStructure, tbSyncTask);
    }

    /**
     * 绑定科组长
     *
     * @param examPaperStructure 试卷结构对象
     */
    public void syncMarkLeader(ExamPaperStructure examPaperStructure) {
        if (examPaperStructure == null) {
            throw ExceptionResultEnum.ERROR.exception("请先设置评卷参数");
        }
        TBSyncTask tbSyncTask = tbSyncTaskService.saveTask(examPaperStructure.getSchoolId(), examPaperStructure.getId(), PushTypeEnum.MARK_LEADER_PUSH, null);
        dataSyncService.syncMarkLeader(examPaperStructure, tbSyncTask);
    }

    /**
     * 同步试卷结构、分组信息、评卷员绑定关系
     *
     * @param examPaperStructure 试卷结构对象
     */
    public void syncMarker(ExamPaperStructure examPaperStructure, GroupInfo groupInfo) {
        if (examPaperStructure == null) {
            throw ExceptionResultEnum.ERROR.exception("未找到同步数据");
        }
        TBSyncTask tbSyncTask = tbSyncTaskService.saveTask(examPaperStructure.getSchoolId(), examPaperStructure.getId(), PushTypeEnum.MARKER_PUSH, JSON.toJSONString(groupInfo));
        dataSyncService.syncMarker(examPaperStructure, tbSyncTask);
    }
}
